# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2019 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE POS coupon views
"""

import six

from corepos.db import model as corepos
from corepos import enum as corepos_enum

from rattail.time import localtime

from .master import CoreOfficeMasterView


class HouseCouponView(CoreOfficeMasterView):
    """
    Master view for house (store) coupons
    """
    model_class = corepos.HouseCoupon
    model_title = "CORE-POS House Coupon"
    url_prefix = '/core-pos/house-coupons'
    route_prefix = 'corepos.house_coupons'
    editable = True

    labels = {
        'coupon_id': "Coupon ID",
        'department_id': "Department Number", # filter
    }

    grid_columns = [
        'coupon_id',
        'description',
        'department',
        'discount_type',
        'discount_value',
        'start_date',
        'end_date',
    ]

    form_fields = [
        'coupon_id',
        'description',
        'limit',
        'start_date',
        'end_date',
        'member_only',
        'auto',
        'department',
        'min_type',
        'min_value',
        'discount_type',
        'discount_value',
        'virtual_only',
    ]

    def configure_grid(self, g):
        super(HouseCouponView, self).configure_grid(g)

        g.set_enum('discount_type', corepos_enum.HOUSE_COUPON_DISCOUNT_TYPE)

        g.set_renderer('start_date', self.render_local_date)
        g.set_renderer('end_date', self.render_local_date)

        g.set_joiner('department', lambda q: q.outerjoin(corepos.Department))
        g.set_filter('department', corepos.Department.name, label="Department Name")
        g.set_sorter('department', corepos.Department.name)

        g.set_sort_defaults('coupon_id', 'desc')

        g.set_link('coupon_id')
        g.set_link('description')

    def render_local_date(self, coupon, field):
        value = getattr(coupon, field)
        if not value:
            return ""
        value = localtime(self.rattail_config, value)
        return six.text_type(value.date())

    def configure_form(self, f):
        super(HouseCouponView, self).configure_form(f)

        if self.creating:
            f.remove('coupon_id')
        else:
            f.set_readonly('coupon_id')

        f.set_enum('member_only', corepos_enum.HOUSE_COUPON_MEMBER_ONLY)

        f.set_renderer('department', self.render_corepos_department)

        f.set_enum('discount_type', corepos_enum.HOUSE_COUPON_DISCOUNT_TYPE)

        f.set_enum('min_type', corepos_enum.HOUSE_COUPON_MINIMUM_TYPE)

        f.set_readonly('department') # TODO: show dropdown for this

        f.set_renderer('start_date', self.render_local_date)
        f.set_readonly('start_date') # TODO
        f.set_type('start_date', 'date_jquery')

        f.set_renderer('end_date', self.render_local_date)
        f.set_readonly('end_date') # TODO
        f.set_type('end_date', 'date_jquery')


def includeme(config):
    HouseCouponView.defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2019 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS member views
"""

import six

from corepos.db import model as corepos

from webhelpers2.html import HTML

from .master import CoreOfficeMasterView


class MemberTypeView(CoreOfficeMasterView):
    """
    Master view for member types
    """
    model_class = corepos.MemberType
    model_title = "CORE-POS Member Type"
    url_prefix = '/core-pos/member-types'
    route_prefix = 'corepos.member_types'

    labels = {
        'id': "ID",
        'ssi': "SSI",
    }

    def configure_grid(self, g):
        super(MemberTypeView, self).configure_grid(g)

        g.set_link('id')
        g.set_link('description')


class MemberView(CoreOfficeMasterView):
    """
    Master view for members
    """
    model_class = corepos.MemberInfo
    model_title = "CORE-POS Member"
    url_prefix = '/core-pos/members'
    route_prefix = 'corepos.members'

    labels = {
        'card_number': "Card No.",
        'email2': "Email 2",
        'ads_ok': "Ads OK",
    }

    grid_columns = [
        'card_number',
        'first_name',
        'last_name',
        'street',
        'city',
        'state',
        'zip',
        'phone',
        'email',
    ]

    form_fields = [
        'card_number',
        'first_name',
        'last_name',
        'other_first_name',
        'other_last_name',
        'street',
        'city',
        'state',
        'zip',
        'phone',
        'email',
        'email2',
        'ads_ok',
        'dates',
    ]

    def configure_grid(self, g):
        super(MemberView, self).configure_grid(g)

        g.set_sort_defaults('card_number')

        g.set_link('card_number')
        g.set_link('first_name')
        g.set_link('last_name')

    def configure_form(self, f):
        super(MemberView, self).configure_form(f)

        f.set_renderer('dates', self.render_member_dates)

    def render_member_dates(self, member, field):
        if not member.dates:
            return ""

        items = []
        for dates in member.dates:
            items.append(HTML.tag('li', c=six.text_type(dates)))
        return HTML.tag('ul', c=items)


def includeme(config):
    MemberTypeView.defaults(config)
    MemberView.defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2019 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE POS master view
"""

from rattail.util import OrderedDict

from webhelpers2.html import tags

from tailbone.views import MasterView
from tailbone_corepos.db import CoreOfficeSession, ExtraCoreOfficeSessions


class CoreOfficeMasterView(MasterView):
    """
    Master base class for Catapult views
    """
    supports_multiple_engines = True
    engine_type_key = 'corepos'

    def get_db_engines(self):
        engines = OrderedDict()
        if self.rattail_config.corepos_engine:
            engines['default'] = self.rattail_config.corepos_engine
        for dbkey in sorted(self.rattail_config.corepos_engines):
            if dbkey != 'default':
                engines[dbkey] = self.rattail_config.corepos_engines[dbkey]
        return engines

    @property
    def Session(self):
        """
        Which session we return will depend on user's "current" engine.
        """
        dbkey = self.get_current_engine_dbkey()

        if dbkey != 'default' and dbkey in ExtraCoreOfficeSessions:
            return ExtraCoreOfficeSessions[dbkey]

        return CoreOfficeSession

    def render_corepos_department(self, obj, field):
        department = getattr(obj, field)
        if not department:
            return ""
        text = "({}) {}".format(department.number, department.name)
        url = self.request.route_url('corepos.departments.view', number=department.number)
        return tags.link_to(text, url)

    def render_corepos_vendor(self, obj, field):
        vendor = getattr(obj, field)
        if not vendor:
            return ""
        text = "({}) {}".format(vendor.abbreviation, vendor.name)
        url = self.request.route_url('corepos.vendors.view', id=vendor.id)
        return tags.link_to(text, url)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2019 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS department views
"""

from corepos.db import model as corepos

from .master import CoreOfficeMasterView


class DepartmentView(CoreOfficeMasterView):
    """
    Base class for department views.
    """
    model_class = corepos.Department
    model_title = "CORE-POS Department"
    url_prefix = '/core-pos/departments'
    route_prefix = 'corepos.departments'

    labels = {
        'dept_see_id': "See ID",
        'modified_by_id': "Modified by ID",
    }

    grid_columns = [
        'number',
        'name',
        'tax',
        'food_stampable',
        'limit',
        'minimum',
        'discount',
        'dept_see_id',
        'modified',
        'modified_by_id',
        'margin',
        'sales_code',
        'member_only',
    ]

    def configure_grid(self, g):
        super(DepartmentView, self).configure_grid(g)

        g.filters['number'].default_active = True
        g.filters['number'].default_verb = 'equal'

        g.filters['name'].default_active = True
        g.filters['name'].default_verb = 'contains'

        # TODO: it should be easier to set only grid header label
        g.set_label('food_stampable', "FS")
        g.filters['food_stampable'].label = "Food Stampable"

        g.set_type('modified', 'datetime_local')

        g.set_sort_defaults('number')

        g.set_link('number')
        g.set_link('name')


def includeme(config):
    DepartmentView.defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2019 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE POS customer views
"""

from corepos.db import model as corepos

from .master import CoreOfficeMasterView


class CustomerView(CoreOfficeMasterView):
    """
    Base class for customer views.
    """
    model_class = corepos.Customer
    model_title = "CORE-POS Customer"
    url_prefix = '/core-pos/customers'
    route_prefix = 'corepos.customers'

    labels = {
        'id': "ID",
        'card_number': "Card No.",
        'person_number': "Person No.",
        'charge_ok': "Charge OK",
        'member_type_id': "Member Type No.",
        'number_of_checks': "Number of Checks",
    }

    grid_columns = [
        'card_number',
        'first_name',
        'last_name',
        'charge_ok',
        'charge_limit',
        'balance',
        'write_checks',
        'purchases',
    ]

    def configure_grid(self, g):
        super(CustomerView, self).configure_grid(g)

        g.filters['first_name'].default_active = True
        g.filters['first_name'].default_verb = 'contains'

        g.filters['last_name'].default_active = True
        g.filters['last_name'].default_verb = 'contains'

        g.set_type('charge_limit', 'currency')
        g.set_type('balance', 'currency')
        g.set_type('purchases', 'currency')

        g.set_sort_defaults('card_number')

        g.set_link('card_number')
        g.set_link('first_name')
        g.set_link('last_name')

    def configure_form(self, f):
        super(CustomerView, self).configure_form(f)

        if self.creating or self.editing:
            f.remove_field('member_info')
            f.remove_field('member_type')
            f.remove_field('last_change')


def includeme(config):
    CustomerView.defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2019 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS transaction views
"""

from corepos.trans.db import model as coretrans

from rattail_corepos.corepos.importing.square import FromSquareToCoreTrans

from .master import CoreOfficeMasterView
from tailbone_corepos.db import CoreTransSession


class TransactionDetailView(CoreOfficeMasterView):
    """
    Master view for transaction details.
    """
    Session = CoreTransSession
    model_class = coretrans.TransactionDetail
    model_title = "CORE-POS Transaction Detail"
    url_prefix = '/corepos/transaction-details'
    route_prefix = 'corepos.transaction_details'
    deletable = True
    bulk_deletable = True
    supports_import_batch_from_file = True

    labels = {
        'store_row_id': "Store Row ID",
        'store_id': "Store ID",
        'pos_row_id': "POS Row ID",
        'transaction_id': "Transaction ID",
        'upc': "UPC",
    }

    grid_columns = [
        'date_time',
        'register_number',
        'transaction_number',
        'card_number',
        'upc',
        'department_number',
        'description',
        'quantity',
        'unit_price',
        'discount',
        'total',
    ]

    def get_bulk_delete_session(self):
        from corepos.trans.db import Session as CoreTransSession

        return CoreTransSession()

    def configure_grid(self, g):
        super(TransactionDetailView, self).configure_grid(g)

        g.set_type('date_time', 'datetime_local')
        g.set_type('quantity', 'quantity')
        g.set_type('unit_price', 'currency')
        g.set_type('discount', 'currency')
        g.set_type('total', 'currency')

        g.set_sort_defaults('date_time', 'desc')

        g.set_label('register_number', "Register")
        g.set_label('transaction_number', "Trans. No.")
        g.set_label('card_number', "Card No.")
        g.set_label('department_number', "Dept. No.")

        g.set_link('upc')
        g.set_link('description')

    def configure_form(self, f):
        super(TransactionDetailView, self).configure_form(f)

        f.set_type('date_time', 'datetime_local')
        f.set_type('quantity', 'quantity')
        f.set_type('unit_price', 'currency')
        f.set_type('discount', 'currency')
        f.set_type('total', 'currency')

    def import_square(self):
        return self.import_batch_from_file(FromSquareToCoreTrans, 'TransactionDetail',
                                           importer_host_title="Square CSV")

    @classmethod
    def defaults(cls, config):
        route_prefix = cls.get_route_prefix()
        url_prefix = cls.get_url_prefix()
        permission_prefix = cls.get_permission_prefix()

        # import from square
        config.add_route('{}.import_square'.format(route_prefix), '{}/import-square'.format(url_prefix))
        config.add_view(cls, attr='import_square', route_name='{}.import_square'.format(route_prefix),
                        permission='{}.import_file'.format(permission_prefix))

        cls._defaults(config)


def includeme(config):
    TransactionDetailView.defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2019 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS product views
"""

from webhelpers2.html import HTML

from corepos.db import model as corepos

from .master import CoreOfficeMasterView


class ProductView(CoreOfficeMasterView):
    """
    Base class for product views.
    """
    model_class = corepos.Product
    model_title = "CORE-POS Product"
    url_prefix = '/core-pos/products'
    route_prefix = 'corepos.products'

    labels = {
        'id': "ID",
        'upc': "UPC",
        'department_number': "Dept. No.",
        'foodstamp': "Food Stamp",
        'unit_of_measure': "Unit of Measure",
        'quantity_enforced': "Qty. Enforced",
        'id_enforced': "ID Enforced",
        'subdepartment_number': "Subdept. No.",
        'default_vendor_id': "Default Vendor ID",
        'current_origin_id': "Current Origin ID",
    }

    grid_columns = [
        'upc',
        'brand',
        'description',
        'size',
        'department',
        'vendor',
        'normal_price',
        'cost',
    ]

    def configure_grid(self, g):
        super(ProductView, self).configure_grid(g)

        g.set_joiner('department', lambda q: q.outerjoin(corepos.Department))
        g.set_sorter('department', corepos.Department.name)

        g.set_joiner('vendor', lambda q: q.outerjoin(corepos.Vendor,
                                                     corepos.Vendor.id == corepos.Product.default_vendor_id))
        g.set_sorter('vendor', corepos.Vendor.name)
        g.set_filter('vendor', corepos.Vendor.name)

        g.filters['upc'].default_active = True
        g.filters['upc'].default_verb = 'equal'

        g.set_type('cost', 'currency')
        g.set_type('normal_price', 'currency')

        g.set_sort_defaults('upc')

        g.set_link('upc')
        g.set_link('brand')
        g.set_link('description')

    def configure_form(self, f):
        super(ProductView, self).configure_form(f)

        f.set_renderer('vendor', self.render_corepos_vendor)

        f.set_renderer('flags', self.render_flags)

        f.set_type('start_date', 'datetime_local')
        f.set_type('end_date', 'datetime_local')
        f.set_type('modified', 'datetime_local')

        f.set_type('normal_price', 'currency')
        f.set_type('group_price', 'currency')
        f.set_type('special_price', 'currency')
        f.set_type('special_group_price', 'currency')
        f.set_type('cost', 'currency')
        f.set_type('deposit', 'currency')

    def render_flags(self, product, field):
        flags = product.flags
        if not flags:
            return ""

        # fetch all flags which are actually defined (supported)
        supported = {}
        for flag in self.Session.query(corepos.ProductFlag):
            supported[flag.bit_number] = flag

        # convert product's flag value to string of bits
        bflags = bin(flags)[2:]   # remove '0b' prefix
        bflags = reversed(bflags) # make bit #1 first in string, etc.

        # create list of items to show each "set" flag
        items = []
        for i, bit in enumerate(bflags, 1):
            if bit == '1':
                flag = supported.get(i)
                if flag:
                    items.append(HTML.tag('li', c=flag.description))
                else:
                    items.append(HTML.tag('li', c="(unsupported bit #{})".format(i)))

        return HTML.tag('ul', c=items)


class ProductFlagView(CoreOfficeMasterView):
    """
    Master view for product flags
    """
    model_class = corepos.ProductFlag
    model_title = "CORE-POS Product Flag"
    url_prefix = '/core-pos/product-flags'
    route_prefix = 'corepos.product_flags'

    def configure_grid(self, g):
        super(ProductFlagView, self).configure_grid(g)

        g.set_link('bit_number')
        g.set_link('description')

    def grid_extra_class(self, flag, i):
        if not flag.active:
            return 'warning'


def includeme(config):
    ProductView.defaults(config)
    ProductFlagView.defaults(config)
